@@ -3,7 +3,7 @@ use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = "v0.0.8";
+our $VERSION = "v0.0.9";
 
 
 
@@ -26,7 +26,18 @@ BoardStreams is an easy way to use WebSockets on the browser and on the server.
 
 =head1 DOCUMENTATION
 
-For the moment, the documentation can be found at: L<https://boardstreams.dev>.
+For now, the documentation can be found at: L<https://boardstreams.dev>.
+
+=head1 ASSOCIATION WITH JAVASCRIPT LIBRARIES
+
+This module release should be used in conjunction with npm package C<boardstreams> version
+L<0.0.8|https://www.npmjs.com/package/boardstreams/v/0.0.8>.
+
+    $ npm install boardstreams@0.0.8 --save
+
+Or:
+
+    <script src="https://unpkg.com/boardstreams@0.0.8"></script>
 
 =head1 NOTIFICATIONS FOR NEW RELEASES
 
