@@ -3,6 +3,8 @@ package BoardStreams::Util;
 use Mojo::Base -strict, -signatures;
 
 use Mojo::JSON qw/ true false /;
+use Mojo::Promise;
+use Mojo::IOLoop;
 use Scalar::Util 'refaddr';
 use List::Util 'any';
 
@@ -14,7 +16,7 @@ our %EXPORT_TAGS = (
     bool => [qw/ true false to_bool /],
 );
 
-our $VERSION = "v0.0.8";
+our $VERSION = "v0.0.9";
 
 sub to_bool :prototype(_) { $_[0] ? true : false }
 
