@@ -27,7 +27,7 @@ use Encode 'encode_utf8';
 
 use experimental 'postderef';
 
-our $VERSION = "v0.0.8";
+our $VERSION = "v0.0.9";
 
 use constant {
     DEFAULT_HEARTBEAT_INTERVAL        => 5,
@@ -245,12 +245,69 @@ sub register ($self, $app, $config) {
         my @answers = $sub->($multi_mode ? \@clone_states : $clone_states[0]);
         @answers = ([@answers]) if not $multi_mode;
 
+        my sub do_notifications ($channel_name, $event_id, $event, $diff) {
+            my $bytes = encode_json({
+                id   => int $event_id,
+                data => {
+                    event => $event,
+                    patch => $diff,
+                },
+            });
+            my $bytes_length = length($bytes);
+
+            if ($bytes_length <= $notify_payload_size_limit) {
+                $db->notify($channel_name, $bytes);
+                return;
+            }
+
+            # my $i = 0;
+            my $ending_bytes_prefix = ":$event_id end: ";
+            my $sent_ending = 0;
+            for (my ($i, $cursor) = (0, 0); ! $sent_ending; $i++) {
+                my $remaining_length = $bytes_length - $cursor;
+                my $bytes_prefix;
+                if (length($ending_bytes_prefix) + $remaining_length <= $notify_payload_size_limit) {
+                    $bytes_prefix = $ending_bytes_prefix;
+                    $sent_ending = 1;
+                } else {
+                    $bytes_prefix = ":$event_id $i: ";
+                }
+
+                my $sublength = $notify_payload_size_limit - length $bytes_prefix;
+                my $substring = $remaining_length >= 0 ? substr($bytes, $cursor, $sublength) : '';
+                $cursor += $sublength;
+
+                my $piece = $bytes_prefix . $substring;
+                $db->notify($channel_name, $piece);
+            }
+        }
+
         my $ea = each_array(@rows, @answers, @orig_states);
         ANSWER:
         while( my ($row, $answer, $orig_state) = $ea->() ) {
 
             my ($event, $new_state, $guard_inc) = @$answer;
             my ($channel_id, $channel_name) = $row->@{qw/ id name /};
+
+            {
+                my @events = ref($event) eq 'REF' ? @$$event : ($event);
+                @events = grep defined, @events;
+                $event = pop @events;
+
+                # for all but last event
+                foreach my $event (@events) {
+                    my ($event_id, $dt) = $db->insert('event_patch',
+                        {
+                            channel_id => $channel_id,
+                            event      => to_json($event),
+                        },
+                        { returning => ['id', 'datetime'] },
+                    )->hash->@{qw/ id datetime /};
+
+                    do_notifications($channel_name, $event_id, $event, undef);
+                }
+            }
+
             next ANSWER unless defined $event or defined $new_state;
             my $diff = defined $new_state ? diff($orig_state, $new_state, noO => 1, noU => 1) : undef;
             $diff = undef if $diff and ! %$diff;
@@ -315,29 +372,7 @@ sub register ($self, $app, $config) {
                 }
             }
 
-            {
-                my $notification = encode_json({
-                    id   => int $event_id,
-                    data => {
-                        event => $event,
-                        patch => $diff,
-                    },
-                });
-
-                if (length $notification > $notify_payload_size_limit) {
-                    my $i = 0;
-                    while (my $len = length $notification) {
-                        my $notification_prefix =
-                            $len > $notify_payload_size_limit ? $i++ :
-                            'end';
-                        substr($notification, 0, 0, ":$notification_prefix: ");
-
-                        $db->notify($channel_name, substr($notification, 0, $notify_payload_size_limit, ''));
-                    }
-                } else {
-                    $db->notify($channel_name, $notification);
-                }
-            }
+            do_notifications($channel_name, $event_id, $event, $diff);
         }
 
         $tx->commit unless $opts{no_txn};
@@ -360,28 +395,39 @@ sub register ($self, $app, $config) {
     });
 
     # NOTE: turn this sub into async later
-    $app->helper("$bs_prefix.send", sub ($c, $data, $identifier) {
+    $app->helper("$bs_prefix.send", sub ($c, $data, $identifier, %opts) {
+        # opts can be: binary
+
         my sub get_max_size {
             return min($c->tx->max_websocket_size, MAX_WEBSOCKET_SIZE);
         }
 
-        my $bytes = encode_json $data;
-        if (length $bytes < get_max_size) {
+        my $bytes = $opts{binary} ? $data : encode_json $data;
+        my $bytes_length = length($bytes);
+        if ($bytes_length < get_max_size) {
             $c->send({binary => $bytes});
             return;
         }
 
-        my $i = 0;
         $identifier =~ s/\:/{}/g;
-        while (length $bytes) {
+        my $ending_bytes_prefix = encode_utf8 ":$identifier end: ";
+        my $sent_ending = 0;
+        for (my ($i, $cursor) = (0, 0); ! $sent_ending; $i++) {
             my $max_size = get_max_size;
-            my $bytes_prefix =
-                length $bytes > $max_size ? $i++ :
-                'end';
-            $bytes_prefix = ":$identifier $bytes_prefix: ";
+            my $remaining_length = $bytes_length - $cursor;
+            my $bytes_prefix;
+            if (length($ending_bytes_prefix) + $remaining_length <= $max_size) {
+                $bytes_prefix = $ending_bytes_prefix;
+                $sent_ending = 1;
+            } else {
+                $bytes_prefix = encode_utf8 ":$identifier $i: ";
+            }
 
-            substr $bytes, 0, 0, encode_utf8($bytes_prefix);
-            my $piece = substr $bytes, 0, $max_size, '';
+            my $sublength = $max_size - length $bytes_prefix;
+            my $substring = $remaining_length >= 0 ? substr($bytes, $cursor, $sublength) : '';
+            $cursor += $sublength;
+
+            my $piece = $bytes_prefix . $substring;
             $c->send({binary => $piece});
         }
     });
